@@ -1,5 +1,5 @@
 import inspect
-from copy import deepcopy
+import re
 from pathlib import Path
 from typing import (
     Any,
@@ -41,9 +41,14 @@ _T = TypeVar('_T')
 
 
 class Smart(Generic[_T]):
-    def __init__(self, _class: Optional[Type[_T]] = None, /, **params: Any) -> None:
-        self._class = _class
-        self._params = params
+    def __init__(
+        self,
+        _class: Optional[Type[_T]] = None,
+        /,
+        **params: Any,
+    ) -> None:
+        self._class: Optional[Type[_T]] = _class
+        self._params: Dict[str, Any] = dict()
 
         self._keyword = 'class'
         self._missing_value = '???'
@@ -59,19 +64,29 @@ class Smart(Generic[_T]):
         self._aliases: Dict[str, str] = dict()
         self._origins: Dict[str, str] = dict()
 
+        for k, v in params.items():
+            self.set(k, v)
+
     @property
     def type(self) -> Optional[Type[_T]]:
         return self._class
 
     @property
     def params(self) -> Dict[str, Any]:
-        return deepcopy(self._params)
+        return self._params.copy()
 
-    def __call__(self, *args: Any, **kwargs: Any) -> _T:
+    def __call__(
+        self,
+        *args: Any,
+        **kwargs: Any,
+    ) -> _T:
         if self._class is None:
             raise AttributeError("Class is not set.")
 
-        params = self._instantiate_dict(self._params, self._location)
+        params = self._instantiate_dict(
+            dictionary=self._params,
+            location=self._location,
+        )
 
         if self._check_overrides:
             check_params_name_override(
@@ -82,7 +97,7 @@ class Smart(Generic[_T]):
 
         params.update(kwargs)
 
-        return self._check_and_instantiate_class(
+        return self._instantiate_class(
             location=self._location,
             cls=self._class,
             args=args,
@@ -97,27 +112,41 @@ class Smart(Generic[_T]):
     def __repr__(self) -> str:
         return str(self)
 
-    def keys(self) -> List[str]:
-        return list(self._params)
+    def keys(
+        self,
+        flatten: bool = False,
+        pattern: Optional[str] = None,
+    ) -> Tuple[str, ...]:
+        keys = flatten_keys(self._params) if flatten else self._params
+        if pattern is not None:
+            return tuple(key for key in keys if re.fullmatch(pattern, key))
+        return tuple(keys)
 
-    def flatten_keys(self) -> List[str]:
-        return flatten_keys(self._params)
+    def items(
+        self,
+        flatten: bool = False,
+        pattern: Optional[str] = None,
+    ) -> Tuple[Tuple[str, Any], ...]:
+        return tuple((k, self.get(k)) for k in self.keys(flatten, pattern))
 
-    def get(self, name: str, init: bool = False, persist: bool = True) -> Any:
+    def get(
+        self,
+        name: str,
+        default: Optional[Any] = None,
+        required: bool = False,
+    ) -> Any:
         dictionary, key = get_nested_dictionary_and_key(
             dictionary=self._params,
             name=name,
+            required=required,
         )
+        return dictionary.get(key, default)
 
-        if init:
-            obj = self._instantiate(dictionary[key], self._location)
-            if persist:
-                dictionary[key] = obj
-            return obj
-
-        return dictionary[key]
-
-    def set(self, name: str, value: Any) -> Any:
+    def set(
+        self,
+        name: str,
+        value: Any,
+    ) -> Any:
         dictionary, key = get_nested_dictionary_and_key(
             dictionary=self._params,
             name=name,
@@ -126,59 +155,82 @@ class Smart(Generic[_T]):
         dictionary[key] = value
         return value
 
-    def pop(self, name: str, init: bool = False) -> Any:
+    def pop(
+        self,
+        name: str,
+        default: Optional[Any] = None,
+        required: bool = False,
+    ) -> Any:
         dictionary, key = get_nested_dictionary_and_key(
             dictionary=self._params,
             name=name,
+            required=required,
         )
-        value = dictionary.pop(key)
-        if init:
-            return self._instantiate(value, self._location)
-        return value
+        return dictionary.pop(key, default)
 
-    def map(self, name: str, function: Callable) -> Any:
+    def map(
+        self,
+        name: str,
+        function: Callable,
+    ) -> Any:
         dictionary, key = get_nested_dictionary_and_key(
             dictionary=self._params,
             name=name,
+            required=True,
         )
         dictionary[key] = value = function(dictionary[key])
         return value
 
-    def update(
+    def update_from(
         self,
         source: Union['Smart', Dict[str, Any], List[str], Path],
         name: Optional[str] = None,
         override: bool = True,
+        required: bool = True,
     ) -> 'Smart':
+        smart: Smart
         if isinstance(source, Smart):
-            self._update_from_smart(
-                smart=source,
-                name=name,
-                override=override,
-            )
+            smart = source
         elif isinstance(source, dict):
-            self._update_from_dict(
-                params=source,
-                name=name,
-                override=override,
-            )
+            smart = Smart(**source)
         elif isinstance(source, list):
-            self._update_from_list(
-                params_list=source,
-                name=name,
-                override=override,
-            )
+            smart = Smart(**dict(map(parse_param, source)))
         elif isinstance(source, Path):
-            self._update_from_path(
-                path=source,
-                name=name,
-                override=override,
-            )
+            smart = Smart(**load_data(source))
         else:
             raise TypeError(f"Source type '{type(source)}' is not supported.")
 
+        if name is None:
+            _flatten_keys = self.keys(flatten=True)
+            for key in smart.keys(flatten=True):
+                if override or not check_key_override(key, _flatten_keys):
+                    self.set(key, smart.get(key, required=True))
+        else:
+            try:
+                self.update_from(
+                    source=smart.get(name, default=dict(), required=required),
+                    override=override,
+                )
+            except Exception as e:
+                raise RuntimeError(f"Cannot update with source name '{name}'. " + ' '.join(e.args))
+
         return self
 
+    def instantiate(
+        self,
+        name: Optional[str] = None,
+        persist: bool = True,
+    ) -> Any:
+        obj = self._instantiate(
+            obj=self.params if name is None else self.get(name),
+            location=self._location,
+        )
+
+        if persist and name is not None:
+            return self.set(name, obj)
+
+        return obj
+
     def representation(
         self,
         skip_defaults: bool = False,
@@ -187,10 +239,13 @@ class Smart(Generic[_T]):
         smart: Smart = Smart()
 
         if merge_params:
-            smart.update(self)
+            smart.update_from(self)
 
-        smart.update(
-            source=self._representation(self._class, skip_default=skip_defaults),
+        smart.update_from(
+            source=self._representation(
+                obj=self._class,
+                skip_default=skip_defaults,
+            ),
             override=False,
         )
 
@@ -199,15 +254,30 @@ class Smart(Generic[_T]):
             missing_value=self._missing_value,
         )
 
-    def register(self, classes: Union[Sequence[str], Mapping[str, str]]) -> 'Smart':
+    def register(
+        self,
+        classes: Union[
+            Sequence[Union[str, Type[Any]]],
+            Mapping[str, str],
+            Mapping[Type[Any], str],
+            Mapping[Union[str, Type[Any]], str],
+        ],
+        prefix: str = '',
+    ) -> 'Smart':
         if self._location:
             msg = f"Classes can only be registered in root Smart object, not in {self._location}."
             raise AttributeError(msg)
 
         if isinstance(classes, Sequence):
-            self._register_classes(classes)
+            self._register_classes(
+                classes=classes,
+                prefix=prefix,
+            )
         elif isinstance(classes, Mapping):
-            self._register_aliases(classes)
+            self._register_aliases(
+                aliases=classes,
+                prefix=prefix,
+            )
         else:
             raise TypeError(f"Register classes type '{type(classes)}' is not supported.")
 
@@ -221,24 +291,30 @@ class Smart(Generic[_T]):
         args = parse_arguments(default_path=path)
 
         if args.path.is_file():
-            self.update(args.path)
+            self.update_from(args.path)
 
-        self.update(args.params)
+        self.update_from(args.params)
 
         if args.dump:
             save_data(
-                data=self.representation(args.skip_defaults, args.merge_params),
+                data=self.representation(
+                    skip_defaults=args.skip_defaults,
+                    merge_params=args.merge_params,
+                ),
                 path=args.path,
             )
         elif args.print:
             if args.print == Print.PARAMS:
                 print_data(
-                    data=self.representation(args.skip_defaults, args.merge_params),
+                    data=self.representation(
+                        skip_defaults=args.skip_defaults,
+                        merge_params=args.merge_params,
+                    ),
                     fmt=args.format,
                 )
             elif args.print == Print.KEYS:
                 print_data(
-                    data=self.flatten_keys(),
+                    data=self.keys(flatten=True),
                     fmt=args.format,
                 )
             else:
@@ -248,8 +324,12 @@ class Smart(Generic[_T]):
 
         return self
 
-    def configurate(
+    def copy(self) -> 'Smart[Type[_T]]':
+        return Smart(self.type, **self.params).setup(smart=self)
+
+    def setup(
         self,
+        smart: Optional['Smart'] = None,
         *,
         keyword: Optional[str] = None,
         missing_value: Optional[str] = None,
@@ -261,117 +341,99 @@ class Smart(Generic[_T]):
     ) -> 'Smart':
         if keyword is not None:
             self._keyword = keyword
+        elif smart is not None:
+            self._keyword = smart._keyword
+
         if missing_value is not None:
             self._missing_value = missing_value
+        elif smart is not None:
+            self._missing_value = smart._missing_value
 
         if location is not None:
             self._location = location
+        elif smart is not None:
+            self._location = smart._location
 
         if check_missings is not None:
             self._check_missings = check_missings
+        elif smart is not None:
+            self._check_missings = smart._check_missings
+
         if check_typings is not None:
             self._check_typings = check_typings
+        elif smart is not None:
+            self._check_typings = smart._check_typings
+
         if check_overrides is not None:
             self._check_overrides = check_overrides
+        elif smart is not None:
+            self._check_overrides = smart._check_overrides
 
         if allow_only_registered is not None:
             if allow_only_registered:
                 self._allow_only_registered = True
             elif self._allow_only_registered:
                 raise AttributeError("Cannot disallow only registered classes if already allowed.")
+        elif smart is not None:
+            self._allow_only_registered = smart._allow_only_registered
+
+        if smart is not None:
+            self._aliases = smart._aliases
+            self._origins = smart._origins
 
         return self
 
-    def _update_from_smart(
-        self,
-        smart: 'Smart',
-        name: Optional[str],
-        override: bool,
-    ) -> None:
-        if name is None:
-            _flatten_keys = self.flatten_keys()
-            for key in smart.flatten_keys():
-                if override or not check_key_override(key, _flatten_keys):
-                    self.set(key, smart.get(key))
-        else:
-            try:
-                self.update(
-                    source=smart.get(name),
-                    override=override,
-                )
-            except Exception as e:
-                raise RuntimeError(f"Cannot update with source name '{name}'. " + ' '.join(e.args))
-
-    def _update_from_dict(
-        self,
-        params: Dict[str, Any],
-        name: Optional[str],
-        override: bool,
-    ) -> None:
-        self._update_from_smart(
-            smart=Smart(**params),
-            name=name,
-            override=override,
-        )
-
-    def _update_from_list(
-        self,
-        params_list: List[str],
-        name: Optional[str],
-        override: bool,
-    ) -> None:
-        smart: Smart = Smart()
-        for param in params_list:
-            key, value = parse_param(param)
-            smart.set(key, value)
-
-        self._update_from_smart(
-            smart=smart,
-            name=name,
-            override=override,
-        )
-
-    def _update_from_path(
-        self,
-        path: Path,
-        name: Optional[str],
-        override: bool,
-    ) -> None:
-        self._update_from_dict(
-            params=load_data(path),
-            name=name,
-            override=override,
-        )
-
     def _instantiate(self, obj: Any, location: str) -> Any:
         if isinstance(obj, dict):
             if self._keyword in obj:
-                return self._instantiate_class(obj, location)
+                return self._instantiate_class_from_dict(
+                    dictionary=obj,
+                    location=location,
+                )
 
-            return self._instantiate_dict(obj, location)
+            return self._instantiate_dict(
+                dictionary=obj,
+                location=location,
+            )
 
         if isinstance(obj, list):
-            return self._instantiate_list(obj, location)
+            return self._instantiate_list(
+                lst=obj,
+                location=location,
+            )
 
         return obj
 
     def _instantiate_dict(self, dictionary: Dict[str, Any], location: str) -> Dict[str, Any]:
         return {
-            key: self._instantiate(value, join_key(location, key))
+            key: self._instantiate(
+                obj=value,
+                location=join_key(location, key),
+            )
             for key, value in dictionary.items()
         }
 
     def _instantiate_list(self, lst: List[Any], location: str) -> List[Any]:
         return [
-            self._instantiate(element, join_key(location, str(index)))
+            self._instantiate(
+                obj=element,
+                location=join_key(location, str(index)),
+            )
             for index, element in enumerate(lst)
         ]
 
-    def _instantiate_class(self, dictionary: Dict[str, Any], location: str) -> Any:
-        kwargs, class_name, option = parse_class(dictionary, self._keyword)
+    def _instantiate_class_from_dict(
+        self,
+        dictionary: Dict[str, Any],
+        location: str,
+    ) -> Any:
+        kwargs, class_name, option = parse_class(
+            dictionary=dictionary,
+            keyword=self._keyword,
+        )
 
         if class_name == self.__class__.__name__:
-            return self._check_and_instantiate_class(
+            return self._instantiate_class(
                 location=location,
                 cls=Smart,
                 kwargs=kwargs,
@@ -386,7 +448,7 @@ class Smart(Generic[_T]):
 
         if option:
             if option == self.__class__.__name__:
-                return self._check_and_instantiate_class(
+                return self._instantiate_class(
                     location=location,
                     cls=Smart,
                     args=(cls,),
@@ -395,13 +457,13 @@ class Smart(Generic[_T]):
             else:
                 raise ValueError(f"Option '{option}' is not supported.")
         else:
-            return self._check_and_instantiate_class(
+            return self._instantiate_class(
                 location=location,
                 cls=cls,
                 kwargs=kwargs,
             )
 
-    def _check_and_instantiate_class(
+    def _instantiate_class(
         self,
         location: str,
         cls: Type[Any],
@@ -433,18 +495,18 @@ class Smart(Generic[_T]):
             raise RuntimeError(f"Error during instantiate {class_location} class.") from e
         else:
             if isinstance(obj, Smart):
-                obj.configurate(
-                    keyword=self._keyword,
-                    missing_value=self._missing_value,
+                obj.setup(
+                    smart=self,
                     location=location,
-                    check_missings=self._check_missings,
-                    check_typings=self._check_typings,
-                    check_overrides=self._check_overrides,
                 )
 
             return obj
 
-    def _representation(self, obj: Any, skip_default: bool = False) -> Dict[str, Any]:
+    def _representation(
+        self,
+        obj: Any,
+        skip_default: bool = False,
+    ) -> Dict[str, Any]:
         representation: Dict[str, Any] = dict()
         signature = inspect.signature(obj)
 
@@ -472,7 +534,10 @@ class Smart(Generic[_T]):
 
                     representation[name] = {
                         self._keyword: keyword,
-                        **self._representation(param_type, skip_default),
+                        **self._representation(
+                            obj=param_type,
+                            skip_default=skip_default,
+                        ),
                     }
                 elif default is not inspect.Parameter.empty and skip_default:
                     continue
@@ -486,23 +551,45 @@ class Smart(Generic[_T]):
                         keyword = self._aliases.get(keyword, keyword)
                         representation[name] = {
                             self._keyword: keyword,
-                            **self._representation(annotation, skip_default),
+                            **self._representation(
+                                obj=annotation,
+                                skip_default=skip_default,
+                            ),
                         }
                 else:
                     representation[name] = self._missing_value
 
         return representation
 
-    def _register_classes(self, classes: Sequence[str]) -> None:
-        self._register_aliases({item: item for item in classes})
+    def _register_classes(
+        self,
+        classes: Sequence[Union[str, Type[Any]]],
+        prefix: str = '',
+    ) -> None:
+        self._register_aliases(
+            aliases={cls: cls if isinstance(cls, str) else get_class_name(cls) for cls in classes},
+            prefix=prefix,
+        )
 
-    def _register_aliases(self, aliases: Mapping[str, str]) -> None:
+    def _register_aliases(
+        self,
+        aliases: Union[
+            Mapping[str, str],
+            Mapping[Type[Any], str],
+            Mapping[Union[str, Type[Any]], str],
+        ],
+        prefix: str = '',
+    ) -> None:
         for origin, alias in aliases.items():
+            origin = origin if isinstance(origin, str) else inspect.formatannotation(origin)
+
             if origin in self._aliases:
                 raise ValueError(f"Origin '{origin}' has been overridden.")
 
             if alias in self._origins:
                 raise ValueError(f"Alias '{alias}' has been overridden.")
 
+            alias = join_key(prefix, alias)
+
             self._aliases[origin] = alias
             self._origins[alias] = origin
