@@ -14,6 +14,7 @@ def get_nested_dictionary_and_key(
     dictionary: Dict[str, Any],
     name: str,
     set_mode: bool = False,
+    required: bool = False,
 ) -> Tuple[Dict[str, Any], str]:
     *nested_keys, last_key = name.split(_KEY_SEPARATOR)
 
@@ -34,7 +35,7 @@ def get_nested_dictionary_and_key(
 
         dictionary = dictionary[key]
 
-    if not set_mode and last_key not in dictionary:
+    if required and last_key not in dictionary:
         raise KeyError(f"Param '{last_key}' is not in dictionary.")
 
     return dictionary, last_key
@@ -99,9 +100,9 @@ def import_class(name: str) -> type:
 
 def get_class_name(obj: Any) -> str:
     try:
-        return obj.__name__
+        return obj.__qualname__
     except AttributeError:
-        return obj.__class__.__name__
+        return obj.__class__.__qualname__
 
 
 def get_hints(signature: inspect.Signature) -> Dict[str, Any]:
