@@ -4,7 +4,7 @@ from typing import Any, Dict, List, Tuple, Union
 from unittest.mock import Mock, patch
 
 from smartparams.smart import Smart
-from tests.custom_classes import Class
+from tests.custom_classes import Class, ClassChild
 from tests.unit import UnitCase
 
 
@@ -98,7 +98,7 @@ class TestSmartAccessCase(UnitCase):
             smart: Smart = deepcopy(self.smart)
 
             with self.subTest(msg=msg):
-                smart.update(source)
+                smart.update_from(source)
 
                 self.assertTrue('arg1' in smart.params)
                 self.assertTrue('arg2' in smart.params)
@@ -123,7 +123,7 @@ class TestSmartAccessCase(UnitCase):
             smart: Smart = deepcopy(self.smart)
 
             with self.subTest(msg=msg):
-                smart.update(source, override=False)
+                smart.update_from(source, override=False)
 
                 self.assertTrue('arg1' in smart.params)
                 self.assertTrue('arg2' in smart.params)
@@ -134,10 +134,18 @@ class TestSmartAccessCase(UnitCase):
                 self.assertEqual('a32', smart.params['arg3']['arg32'])
                 self.assertEqual({'a4': 'aa4'}, smart.params['arg4'])
 
+    def test_update__not_required(self) -> None:
+        dictionary = {'arg1': {'nested1': 'aa1'}}
+        expected = deepcopy(self.smart.params)
+
+        self.smart.update_from(dictionary, name='arg2', required=False)
+
+        self.assertEqual(expected, self.smart.params)
+
     def test_update__with_name(self) -> None:
         dictionary = {'arg': {'nested': ['arg3.arg31=argument31', 'arg1={"nested1": "argument1"}']}}
 
-        self.smart.update(dictionary, name='arg.nested')
+        self.smart.update_from(dictionary, name='arg.nested')
 
         self.assertTrue('arg1' in self.smart.params)
         self.assertTrue('arg2' in self.smart.params)
@@ -150,17 +158,33 @@ class TestSmartAccessCase(UnitCase):
     def test_update__with_name_error(self) -> None:
         dictionary = {'arg': {'nested': ['arg3.arg31 argument31']}}
 
-        self.assertRaises(RuntimeError, self.smart.update, dictionary, name='arg.nested')
+        self.assertRaises(RuntimeError, self.smart.update_from, dictionary, name='arg.nested')
 
     def test_update__unknown(self) -> None:
         source = tuple()  # type: ignore
 
-        self.assertRaises(TypeError, self.smart.update, source)
+        self.assertRaises(TypeError, self.smart.update_from, source)
 
-    def test_flatten_keys(self) -> None:
-        value = self.smart.flatten_keys()
+    def test_keys__flatten(self) -> None:
+        value = self.smart.keys(flatten=True)
 
-        self.assertListEqual(['arg1', 'arg2', 'arg3.arg31', 'arg3.arg32'], value)
+        self.assertTupleEqual(('arg1', 'arg2', 'arg3.arg31', 'arg3.arg32'), value)
+
+    def test_keys__flatten_pattern(self) -> None:
+        value = self.smart.keys(flatten=True, pattern='arg[1,3].*')
+
+        self.assertTupleEqual(('arg1', 'arg3.arg31', 'arg3.arg32'), value)
+
+    def test_items(self) -> None:
+        expected = (
+            ('arg1', 'arg1'),
+            ('arg2', ['arg2']),
+            ('arg3', {'arg31': 'a31', 'arg32': 'a32'}),
+        )
+
+        actual = self.smart.items()
+
+        self.assertTupleEqual(expected, actual)
 
 
 class TestSmartInstantiateCase(UnitCase):
@@ -174,58 +198,20 @@ class TestSmartInstantiateCase(UnitCase):
         )
         self.smart: Smart = Smart(**self.params)
 
-    def test_get__init_smart(self) -> None:
-        obj = self.smart.get('smart', init=True)
+    def test_instantiate__persist(self) -> None:
+        obj = self.smart.instantiate('object')
 
-        self.assertIsInstance(obj, Smart)
-        self.assertIs(obj.type, Class)
-        self.assertTrue('smart' in self.smart.params)
-
-    def test_get__init_object(self) -> None:
-        obj = self.smart.get('object', init=True)
-
-        self.assertIsInstance(obj, Class)
-        self.assertTrue('object' in self.smart.params)
-
-    def test_get__init_value(self) -> None:
-        obj = self.smart.get('value', init=True)
-
-        self.assertIsInstance(obj, int)
-        self.assertTrue('value' in self.smart.params)
-
-    def test_get__init_persist(self) -> None:
-        obj = self.smart.get('object', init=True, persist=True)
-
-        self.assertIs(self.smart.get('object', init=True, persist=False), obj)
+        self.assertIs(self.smart.get('object'), obj)
         self.assertIsInstance(self.smart.get('object'), Class)
         self.assertTrue('object' in self.smart.params)
 
-    def test_get__init_not_persist(self) -> None:
-        obj = self.smart.get('object', persist=False)
+    def test_instantiate__not_persist(self) -> None:
+        obj = self.smart.instantiate('object', persist=False)
 
-        self.assertIsNot(self.smart.get('object', init=True, persist=False), obj)
+        self.assertIsNot(self.smart.get('object'), obj)
         self.assertIsInstance(self.smart.get('object'), dict)
         self.assertTrue('object' in self.smart.params)
 
-    def test_pop__init_smart(self) -> None:
-        obj = self.smart.pop('smart', init=True)
-
-        self.assertIsInstance(obj, Smart)
-        self.assertIs(obj.type, Class)
-        self.assertFalse('smart' in self.smart.params)
-
-    def test_pop__init_object(self) -> None:
-        obj = self.smart.pop('object', init=True)
-
-        self.assertIsInstance(obj, Class)
-        self.assertFalse('object' in self.smart.params)
-
-    def test_pop__init_value(self) -> None:
-        obj = self.smart.pop('value', init=True)
-
-        self.assertIsInstance(obj, int)
-        self.assertFalse('value' in self.smart.params)
-
     def test_instantiate__any(self) -> None:
         data = "string"
 
@@ -304,6 +290,21 @@ class TestSmartInstantiateCase(UnitCase):
 
         self.assertRaises(ImportError, self.smart._instantiate, obj, '')
 
+    def test_copy(self) -> None:
+        self.smart.setup(
+            location='__mock__',
+            allow_only_registered=True,
+        )
+
+        smart = self.smart.copy()
+
+        self.assertIsNot(self.smart, smart)
+        self.assertEqual(self.smart.type, smart.type)
+        self.assertEqual(self.smart.params, smart.params)
+        self.assertEqual(self.smart._location, smart._location)
+        self.assertEqual(self.smart._allow_only_registered, smart._allow_only_registered)
+        self.assertEqual(self.smart._missing_value, smart._missing_value)
+
 
 class TestRegisterCase(UnitCase):
     def setUp(self) -> None:
@@ -318,6 +319,23 @@ class TestRegisterCase(UnitCase):
         self.assertEqual({'origin.1': 'origin.1', 'origin.2': 'origin.2'}, self.smart._aliases)
         self.assertEqual({'origin.1': 'origin.1', 'origin.2': 'origin.2'}, self.smart._origins)
 
+    def test_register__with_prefix_with_class(self) -> None:
+        aliases = [Class, ClassChild]
+        expected_aliases = {
+            'tests.custom_classes.Class': 'some.classes.Class',
+            'tests.custom_classes.ClassChild': 'some.classes.ClassChild',
+        }
+        expected_origins = {
+            'some.classes.Class': 'tests.custom_classes.Class',
+            'some.classes.ClassChild': 'tests.custom_classes.ClassChild',
+        }
+
+        self.smart.register(aliases, prefix='some.classes')
+
+        self.assertFalse(self.smart._allow_only_registered)
+        self.assertEqual(expected_aliases, self.smart._aliases)
+        self.assertEqual(expected_origins, self.smart._origins)
+
     def test_register__aliases(self) -> None:
         aliases = {'origin.1': 'alias.1', 'origin.2': 'alias.2'}
 
@@ -350,25 +368,75 @@ class TestRegisterCase(UnitCase):
         self.assertRaises(AttributeError, self.smart.register, Mock())
 
 
-class TestConfigurationCase(UnitCase):
+class TestSetupCase(UnitCase):
     def setUp(self) -> None:
         self.smart: Smart = Smart()
 
-    def test_configurate__allow_only_registered_true(self) -> None:
-        self.smart.configurate(
+    def test_setup__manual(self) -> None:
+        self.smart.setup(
+            keyword='__CLASS__',
+            missing_value='?',
+            location='__mock__',
+            check_missings=True,
+            check_typings=True,
+            check_overrides=True,
+            allow_only_registered=True,
+        )
+
+        self.assertEqual('__CLASS__', self.smart._keyword)
+        self.assertEqual('?', self.smart._missing_value)
+        self.assertEqual('__mock__', self.smart._location)
+        self.assertTrue(self.smart._check_missings)
+        self.assertTrue(self.smart._check_typings)
+        self.assertTrue(self.smart._check_overrides)
+        self.assertTrue(self.smart._allow_only_registered)
+
+    def test_setup__smart(self) -> None:
+        smart: Smart = Smart().setup(
+            location='__mock__',
+            keyword='__class__',
+            allow_only_registered=True,
+        )
+
+        self.smart.setup(
+            smart=smart,
+        )
+
+        self.assertEqual('__class__', self.smart._keyword)
+        self.assertEqual('__mock__', self.smart._location)
+        self.assertTrue(self.smart._allow_only_registered)
+
+    def test_setup__mix(self) -> None:
+        smart: Smart = Smart().setup(
+            location='__mock__',
+            keyword='__class__',
+            allow_only_registered=True,
+        )
+
+        self.smart.setup(
+            smart=smart,
+            keyword='__CLASS__',
+        )
+
+        self.assertEqual('__CLASS__', self.smart._keyword)
+        self.assertEqual('__mock__', self.smart._location)
+        self.assertTrue(self.smart._allow_only_registered)
+
+    def test_setup__allow_only_registered_true(self) -> None:
+        self.smart.setup(
             allow_only_registered=True,
         )
 
         self.assertTrue(self.smart._allow_only_registered)
 
-    def test_configurate__allow_only_registered_false(self) -> None:
-        self.smart.configurate(
+    def test_setup__allow_only_registered_false(self) -> None:
+        self.smart.setup(
             allow_only_registered=False,
         )
 
         self.assertFalse(self.smart._allow_only_registered)
 
-    def test_configurate__allow_only_registered_false_raise(self) -> None:
+    def test_setup__allow_only_registered_false_raise(self) -> None:
         self.smart._allow_only_registered = True
 
-        self.assertRaises(AttributeError, self.smart.configurate, allow_only_registered=False)
+        self.assertRaises(AttributeError, self.smart.setup, allow_only_registered=False)
