@@ -31,7 +31,7 @@ class TestGetNestedDictionaryAndKey(UnitCase):
 
         self.assertRaises(KeyError, get_nested_dictionary_and_key, dictionary=self.dict, name=name)
 
-    def test_get_nested_dictionary_and_key__set_mode_false(self) -> None:
+    def test_get_nested_dictionary_and_key__required_true(self) -> None:
         name = 'arg3.missing'
 
         self.assertRaises(
@@ -39,7 +39,7 @@ class TestGetNestedDictionaryAndKey(UnitCase):
             get_nested_dictionary_and_key,
             dictionary=self.dict,
             name=name,
-            set_mode=False,
+            required=True,
         )
 
     def test_get_nested_dictionary_and_key__is_not_dictionary(self) -> None:
@@ -107,8 +107,8 @@ class TestGetClassName(UnitCase):
             (123, 'int'),
             (type, 'type'),
             (None, 'NoneType'),
-            (lambda: ..., '<lambda>'),
-            ((i for i in range(1)), '<genexpr>'),
+            (lambda: ..., 'TestGetClassName.test_get_class_name.<locals>.<lambda>'),
+            ((i for i in range(1)), 'TestGetClassName.test_get_class_name.<locals>.<genexpr>'),
         )
 
         for cls, expected in test_cases:
